package grafos;

import com.github.javaparser.ast.Node;
import com.github.javaparser.ast.stmt.EmptyStmt;

import java.util.*;

public class CFG {
	private final List<Node> nodes = new ArrayList<Node>();
	private final Map<Node, List<Node>> edges = new HashMap<Node, List<Node>>();
	private Node beginNode, endNode;

	public void addNode(Node stmt) {
		nodes.add(stmt);
		System.out.println("NODO: " + node2str(stmt));
	}

	public void connect(int begin, int end) {
		connect(nodes.get(begin), nodes.get(end));
	}

	public void connect(Node begin, Node end) {
		if (edges.containsKey(begin) && !edges.get(begin).contains(end)) {
			edges.get(begin).add(end);
		} else {
			List<Node> dest = new ArrayList<Node>();
			dest.add(end);
			edges.put(begin, dest);
		}
	}

	public void connect(Collection<Node> begin, Node end) {
		for (Node n : begin)
			connect(n, end);
	}

	public Node beginNode() {
		if (beginNode == null) {
			beginNode = new EmptyStmt();
			nodes.add(beginNode);
		}
		return beginNode;
	}

	public Node endNode() {
		if (endNode == null) {
			endNode = new EmptyStmt();
			nodes.add(endNode);
		}
		return endNode;
	}

	public List<String> toStringList() {
		List<String> res = new LinkedList<String>();
		for (Map.Entry<Node, List<Node>> e : edges.entrySet()) {
			for (Node dest : e.getValue()) {
				res.add(edge2str(e.getKey(), dest));
			}
		}
		return res;
	}

	private String edge2str(Node a, Node b) {
		return node2str(a) + "->" + node2str(b);
	}

	private String node2str(Node s) {
		if (s == beginNode)
			return "Start";
		if (s == endNode)
			return "Stop";
		int index = nodes.indexOf(s);
		return "\"(" + index + ") " + s.toString().replace("\"", "\\\"") + "\"";
	}
}
